@@ -29,12 +29,16 @@ from time import sleep
 from PyQt4 import QtCore
 
 from friend import Friend
-from lib import search_contact, banner_notification
+from lib import search_contact_by_number, banner_notification
+from lib import logger
 from history import insert_sms_in_history
+from config import config
+from controller import Controller
 
 recv_sms_q = Queue.Queue()
 send_sms_q = Queue.Queue()
 sms_history_q = Queue.Queue()
+incoming_call_q = Queue.Queue()
 
 
 class Scheduler(QtCore.QThread):
@@ -42,18 +46,32 @@ class Scheduler(QtCore.QThread):
         QtCore.QThread.__init__(self)
         self.friend_list = []
         self.parent = parent
+        self.must_run = False
 
     def run(self):
-        while True:
+        while self.parent.bonjour_auth_user == '':
+            logger.debug("Waiting selection of an authorized bonjour contact")
             sleep(1)
+
+        bonjour_auth_username = str(self.parent.bonjour_auth_user)
+        self.auth_user = {bonjour_auth_username:
+                            self.parent.bonjour_users[bonjour_auth_username]}
+
+        self.add_starting_friends()
+
+        if config.usecontroler == QtCore.Qt.Checked:
+            # Startup controller
+            self.start_controller()
+
+        self.must_run = True
+        while self.must_run:
+            sleep(0.1)
             if not recv_sms_q.empty():
                 new_sms = recv_sms_q.get()
                 status = self.sms_received(new_sms['phone_number'],
                               new_sms['message'])
                 if status:
                     recv_sms_q.task_done()
-                else:
-                    recv_sms_q.put(new_sms)
             if not send_sms_q.empty():
                 new_sms = send_sms_q.get()
                 self.send_sms(new_sms['to'],
@@ -64,61 +82,91 @@ class Scheduler(QtCore.QThread):
                 insert_sms_in_history(sms)
                 sms_history_q.task_done()
 
+    def stop_controller(self):
+        self.controller.close()
+
+    def start_controller(self):
+        self.controller = Controller(self.auth_user, self.parent)
+        self.controller.start()
+        self.friend_list.append(self.controller)
+
+    def add_starting_friends(self):
+        friend_numbers = config.startup_friends
+        for number in friend_numbers:
+            logger.debug("This is a starting friend: %s" % number)
+            fullname = search_contact_by_number(number)
+            # Save it !
+            while self.parent.bonjour_auth_user == '':
+                logger.debug("Waiting selection of an authorized bonjour contact")
+                sleep(1)
+            bonjour_auth_username = str(self.parent.bonjour_auth_user)
+            auth_user = {bonjour_auth_username:
+                            self.parent.bonjour_users[bonjour_auth_username]}
+            new_friend = Friend(fullname, number, auth_user)
+            new_friend.favorite = True
+            # Add to friend list in table model
+            self.parent.central_widget.friends_list.emit(QtCore.SIGNAL("add_friend"), new_friend)
+            # append to friend list
+            self.friend_list.append(new_friend)
+            # Register it on bonjour
+            new_friend.start()
+
     def set_auth(self, bonjour_auth_user):
         bonjour_auth_username = str(bonjour_auth_user)
         if bonjour_auth_username:
             auth_user = {bonjour_auth_username:
                      self.parent.bonjour_users[bonjour_auth_username]}
-            print auth_user
+            logger.debug("Bonjour contact which receive message: %s" % auth_user)
             for f in self.friend_list:
                 f.auth_user = auth_user
         else:
             banner_notification("Avahi error, please restart HeySms")
 
     def send_sms(self, to, msg):
-        print "send sms to ", to
-        print "content ", msg
-        name_list = [friend.fullname for friend in self.friend_list]
+        logger.debug("Send sms to: %s" % to)
+        logger.debug("Sms content: %s" % msg)
+        node_list = [friend.node for friend in self.friend_list]
         try:
-            i = name_list.index(to)
+            i = node_list.index(to)
         except ValueError:
             # Impossible ?
-            print "User not find in list"
+            logger.debug("User not find in list: %s" % to)
+            return
         friend = self.friend_list[i]
-        friend.send_sms(msg)
-        sms_history_q.put({'message': msg, 'num': friend.number})
+        controller = friend.send_sms(msg)
+        if not controller:
+            sms_history_q.put({'message': msg, 'num': friend.number})
 
     def sms_received(self, sender, msg):
-        number_list = [friend.number for friend in self.friend_list]
-        print 'sms_received', sender
-        if not sender in number_list:
+        logger.debug("New sms from: %s" % sender)
+        self.number_list = [friend.number for friend in self.friend_list]
+        if not sender in self.number_list:
             # Create a new friend
-            print 'newfriend', sender
-            fullname = search_contact(str(sender))
-            name = "lastname"
-            first_name = "firstname"
+            logger.debug("This is a new friend: %s" % sender)
+            fullname = search_contact_by_number(str(sender))
             number = str(sender)
             # Save it !
             bonjour_auth_username = str(self.parent.bonjour_auth_user)
             auth_user = {bonjour_auth_username:
                             self.parent.bonjour_users[bonjour_auth_username]}
             new_friend = Friend(fullname, number, auth_user)
-            # append to friend il
+            # Add to friend list in table model
+            self.parent.central_widget.friends_list.emit(QtCore.SIGNAL("add_friend"), new_friend)
+            # append to friend list
             self.friend_list.append(new_friend)
             # Register it on bonjour
             new_friend.start()
             friend = new_friend
-
         else:
-            i = number_list.index(sender)
+            i = self.number_list.index(sender)
             friend = self.friend_list[i]
-            print "new sms from an old friend : ", friend.number
+            logger.debug("This is an old friend: %s" % sender)
         # SMS to bonjour
-        print 'send sms to bonjour'
+        logger.debug("Forward sms to bonjour")
         try:
             ret = friend.sms_to_bonjour(msg)
-            print "RET", ret
+            logger.debug("sms_to_bonjour return: %s" % str(ret))
             return ret
         except Exception, e:
-            print "ERROR", e
+            logger.debug("sms_to_bonjour error: %s" % str(e))
             return False
